#ifndef MEM_H
#define MEM_H

#include <stdint.h>

#include "rpcemu.h"

extern uint32_t readmemfl(uint32_t addr);
extern uint32_t readmemfb(uint32_t addr);
extern void writememfb(uint32_t addr, uint8_t val);
extern void writememfl(uint32_t addr, uint32_t val);
extern uint32_t readmemfb(uint32_t addr);
extern void clearmemcache(void);
extern void mem_init(void);
extern void mem_reset(uint32_t ramsize);

extern unsigned long *vraddrl;
extern uint32_t vraddrls[1024],vraddrphys[1024];

static inline uint32_t
readmeml(uint32_t a)
{
	if (unlikely(vraddrl[a >> 12] & 1)) {
		return readmemfl(a);
	}
	return *(const uint32_t *) (a + vraddrl[a >> 12]);
}

static inline uint8_t
readmemb(uint32_t a)
{
	if (unlikely(vraddrl[a >> 12] & 1)) {
		return readmemfb(a);
	}
#ifdef _RPCEMU_BIG_ENDIAN
	return *(const uint8_t *) ((a ^ 3) + vraddrl[a >> 12]);
#else
	return *(const uint8_t *) (a + vraddrl[a >> 12]);
#endif
}

extern unsigned long *vwaddrl;
extern uint32_t vwaddrls[1024],vwaddrphys[1024];

//uint8_t pagedirty[0x1000];

#define HASH(l) (((l)>>2)&0x7FFF)

static inline void
writememl(uint32_t a, uint32_t v)
{
	if (unlikely(vwaddrl[a >> 12] & 3)) {
		writememfl(a, v);
	} else {
		*(uint32_t *) (a + vwaddrl[a >> 12]) = v;
	}
}

static inline void
writememb(uint32_t a, uint8_t v)
{
	if (unlikely(vwaddrl[a >> 12] & 3)) {
		writememfb(a, v);
	} else {
#ifdef _RPCEMU_BIG_ENDIAN
		*(uint8_t *) ((a ^ 3) + vwaddrl[a >> 12]) = v;
#else
		*(uint8_t *) (a + vwaddrl[a >> 12]) = v;
#endif
	}
}

#define ROMSIZE (8*1024*1024)

extern uint32_t *ram,*ram2,*rom,*vram;
extern uint8_t *ramb,*romb,*vramb;

extern uint32_t tlbcache[0x100000];
#define translateaddress(addr,rw,prefetch) ((/*!((addr)&0xFC000000) && */!(tlbcache[((addr)>>12)/*&0x3FFF*/]&0xFFF))?(tlbcache[(addr)>>12]|((addr)&0xFFF)):translateaddress2(addr,rw,prefetch))

extern int mmu,memmode;

extern void cacheclearpage(uint32_t a);

#endif /* MEM_H */
